@@ -10,6 +10,7 @@
 
 from datalad.tests.utils import known_failure_v6
 from datalad.tests.utils import known_failure_direct_mode
+from datalad.tests.utils import skip_if_on_windows
 
 
 import re
@@ -18,9 +19,10 @@ from six.moves import StringIO
 from mock import patch
 
 import datalad
-from ..main import main
+from ..main import main, fail_with_short_help
 from datalad import __version__
 from datalad.cmd import Runner
+from datalad.ui.utils import get_console_width
 from datalad.api import create
 from datalad.utils import chpwd
 from datalad.tests.utils import with_tempfile
@@ -28,6 +30,7 @@ from datalad.tests.utils import assert_equal, assert_raises, in_, ok_startswith
 from datalad.tests.utils import assert_in
 from datalad.tests.utils import assert_re_in
 from datalad.tests.utils import assert_not_in
+from datalad.tests.utils import slow
 
 
 def run_main(args, exit_code=0, expect_stderr=False):
@@ -50,8 +53,8 @@ def run_main(args, exit_code=0, expect_stderr=False):
     with patch('sys.stderr', new_callable=StringIO) as cmerr:
         with patch('sys.stdout', new_callable=StringIO) as cmout:
             with assert_raises(SystemExit) as cm:
-                main(args)
-            assert_equal(cm.exception.code, exit_code)  # exit code must be 0
+                main(["datalad"] + list(args))
+            assert_equal(cm.exception.code, exit_code)
             stdout = cmout.getvalue()
             stderr = cmerr.getvalue()
             if expect_stderr is False:
@@ -89,20 +92,19 @@ def test_help_np():
     # but order is still not guaranteed (dict somewhere)! TODO
     # see https://travis-ci.org/datalad/datalad/jobs/80519004
     # thus testing sets
-    assert_equal(set(sections),
-                 {'Commands for dataset operations',
-                  'Commands for meta data handling',
-                  'Miscellaneous commands',
-                  'General information',
-                  'Global options',
-                  'Plumbing commands'})
+    for s in {'Commands for dataset operations',
+              'Commands for metadata handling',
+              'Miscellaneous commands',
+              'General information',
+              'Global options',
+              'Plumbing commands',
+              'Plugins'}:
+        assert_in(s, sections)
 
     # none of the lines must be longer than 80 chars
     # TODO: decide on   create-sibling and possibly
     # rewrite-urls
-    import shutil
-    accepted_width = shutil.get_terminal_size()[0] \
-        if hasattr(shutil, 'get_terminal_size') else 80
+    accepted_width = get_console_width()
 
     long_lines = ["%d %s" % (len(l), l) for l in stdout.split('\n')
                   if len(l) > accepted_width and
@@ -121,7 +123,7 @@ def test_usage_on_insufficient_args():
 
 
 def test_subcmd_usage_on_unknown_args():
-    stdout, stderr = run_main(['get', '--murks'], exit_code=1)
+    stdout, stderr = run_main(['get', '--murks'], exit_code=1, expect_stderr=True)
     in_('get', stdout)
 
 
@@ -141,7 +143,7 @@ def check_incorrect_option(opts, err_str):
     # checked to meet its constraints.
     # But sys.argv[0] actually isn't used by main at all. It simply doesn't
     # matter what's in there. The only thing important to pass here is `opts`.
-    stdout, stderr = run_main(('datalad',) + opts, expect_stderr=True, exit_code=2)
+    stdout, stderr = run_main(opts, expect_stderr=True, exit_code=2)
     out = stdout + stderr
     assert_in("usage: ", out)
     assert_re_in(err_str, out, match=False)
@@ -149,15 +151,16 @@ def check_incorrect_option(opts, err_str):
 
 def test_incorrect_options():
     # apparently a bit different if following a good one so let's do both
-    err_invalid = "error: (invalid|too few arguments)"
+    err_invalid = "error: (invalid|too few arguments|unrecognized argument)"
     yield check_incorrect_option, ('--buga',), err_invalid
     yield check_incorrect_option, ('--dbg', '--buga'), err_invalid
 
-    err_insufficient = err_invalid # "specify"
+    err_insufficient = err_invalid  # "specify"
     yield check_incorrect_option, ('--dbg',), err_insufficient
     yield check_incorrect_option, tuple(), err_insufficient
 
 
+@skip_if_on_windows
 def test_script_shims():
     runner = Runner()
     for script in [
@@ -184,32 +187,77 @@ def test_script_shims():
                      get_numeric_portion(version))
 
 
+@slow  # 11.2591s
 @with_tempfile(mkdir=True)
 def test_cfg_override(path):
     with chpwd(path):
         # control
-        out, err = Runner()('datalad plugin wtf', shell=True)
+        out, err = Runner()('datalad wtf -s some', shell=True)
         assert_not_in('datalad.dummy: this', out)
         # ensure that this is not a dataset's cfg manager
         assert_not_in('datalad.dataset.id', out)
         # env var
-        out, err = Runner()('DATALAD_DUMMY=this datalad plugin wtf', shell=True)
+        from datalad.utils import on_windows
+        if on_windows:
+            cmd_str = 'set DATALAD_DUMMY=this&& datalad wtf -s some'
+        else:
+            cmd_str = 'DATALAD_DUMMY=this datalad wtf -s some'
+        out, err = Runner()(cmd_str, shell=True)
         assert_in('datalad.dummy: this', out)
         # cmdline arg
-        out, err = Runner()('datalad -c datalad.dummy=this plugin wtf', shell=True)
+        out, err = Runner()('datalad -c datalad.dummy=this wtf -s some', shell=True)
         assert_in('datalad.dummy: this', out)
 
         # now create a dataset in the path. the wtf plugin will switch to
         # using the dataset's config manager, which must inherit the overrides
         create(dataset=path)
         # control
-        out, err = Runner()('datalad plugin wtf', shell=True)
+        out, err = Runner()('datalad wtf -s some', shell=True)
         assert_not_in('datalad.dummy: this', out)
         # ensure that this is a dataset's cfg manager
         assert_in('datalad.dataset.id', out)
         # env var
-        out, err = Runner()('DATALAD_DUMMY=this datalad plugin wtf', shell=True)
+        if on_windows:
+            cmd_str = 'set DATALAD_DUMMY=this&& datalad wtf -s some'
+        else:
+            cmd_str = 'DATALAD_DUMMY=this datalad wtf -s some'
+        out, err = Runner()(cmd_str, shell=True)
         assert_in('datalad.dummy: this', out)
         # cmdline arg
-        out, err = Runner()('datalad -c datalad.dummy=this plugin wtf', shell=True)
+        out, err = Runner()('datalad -c datalad.dummy=this wtf -s some', shell=True)
         assert_in('datalad.dummy: this', out)
+
+
+def test_fail_with_short_help():
+    out = StringIO()
+    with assert_raises(SystemExit) as cme:
+        fail_with_short_help(exit_code=3, out=out)
+    assert_equal(cme.exception.code, 3)
+    assert_equal(out.getvalue(), "")
+
+    out = StringIO()
+    with assert_raises(SystemExit) as cme:
+        fail_with_short_help(msg="Failed badly", out=out)
+    assert_equal(cme.exception.code, 1)
+    assert_equal(out.getvalue(), "error: Failed badly\n")
+
+    # Suggestions, hint, etc
+    out = StringIO()
+    with assert_raises(SystemExit) as cme:
+        fail_with_short_help(
+            msg="Failed badly",
+            known=["mother", "mutter", "father", "son"],
+            provided="muther",
+            hint="You can become one",
+            exit_code=0,  # noone forbids
+            what="parent",
+            out=out)
+    assert_equal(cme.exception.code, 0)
+    assert_equal(out.getvalue(),
+                 "error: Failed badly\n"
+                 "datalad: Unknown parent 'muther'.  See 'datalad --help'.\n\n"
+                 "Did you mean one of these?\n"
+                 "        mutter\n"
+                 "        mother\n"
+                 "        father\n"
+                 "Hint: You can become one\n")
