@@ -96,7 +96,8 @@ def _makeds(path, levels, ds=None, max_leading_dirs=2):
     fn = opj(path, "file%d.dat" % random.randint(1, 1000))
     with open(fn, 'w') as f:
         f.write(fn)
-    repo.add(fn, git=True, commit=True, msg="Added %s" % fn, _datalad_msg=True)
+    repo.add(fn, git=True)
+    repo.commit(msg="Added %s" % fn)
 
     yield path
 
