@@ -21,8 +21,11 @@ from ..base import DownloadError
 from ..base import IncompleteDownloadError
 from ..base import BaseDownloader
 from ..credentials import UserPassword
+from ..credentials import Token
+from ..credentials import LORIS_Token
 from ..http import HTMLFormAuthenticator
 from ..http import HTTPDownloader
+from ..http import HTTPBearerTokenAuthenticator
 from ...support.network import get_url_straight_filename
 from ...tests.utils import with_fake_cookies_db
 from ...tests.utils import skip_if_no_network
@@ -34,7 +37,9 @@ try:
     if PY3:
         raise ImportError("Not yet ready apparently: https://travis-ci.org/datalad/datalad/jobs/111659666")
     import httpretty
-except ImportError:
+except (ImportError, AttributeError):
+    # Attribute Error happens with newer httpretty and older ssl module
+    # https://github.com/datalad/datalad/pull/2623
     class NoHTTPPretty(object):
        __bool__ = __nonzero__ = lambda s: False
        activate = lambda s, t: t
@@ -105,6 +110,12 @@ def test_HTTPDownloader_basic(toppath, topurl):
     assert_equal(downloaded_path, tfpath)
     ok_file_has_content(tfpath, 'abc')
 
+    # Fail with an informative message if we're downloading into a directory
+    # and the file name can't be determined from the URL.
+    with assert_raises(DownloadError) as cm:
+        download(topurl, toppath)
+    assert_in("File name could not be determined", str(cm.exception))
+
     # Some errors handling
     # XXX obscure mocking since impossible to mock write alone
     # and it still results in some warning being spit out
@@ -282,10 +293,15 @@ def test_get_status_from_headers():
 
 class FakeCredential1(UserPassword):
     """Credential to test scenarios."""
-    _fixed_credentials = [
-        {'user': 'testlogin', 'password': 'testpassword'},
-        {'user': 'testlogin2', 'password': 'testpassword2'},
-        {'user': 'testlogin2', 'password': 'testpassword3'}]
+    # to be reusable, and not leak across tests,
+    # we should get _fixed_credentials per instance
+    def __init__(self, *args, **kwargs):
+        super(FakeCredential1, self).__init__(*args, **kwargs)
+        self._fixed_credentials = [
+            {'user': 'testlogin', 'password': 'testpassword'},
+            {'user': 'testlogin2', 'password': 'testpassword2'},
+            {'user': 'testlogin2', 'password': 'testpassword3'}
+        ]
     def is_known(self):
         return True
     def __call__(self):
@@ -379,13 +395,26 @@ def test_auth_but_no_cred(keyring):
     assert_equal(downloader.credential.get('password'), 'testpassword')
 
 
+@with_testsui(responses=['yes'])  # will request to reentry it
+def test_authfail404_interactive():
+    # we will firsts get 'failed' but then real 404 when trying new password
+    check_httpretty_authfail404(['failed', '404'])
+
+
+@with_testsui(interactive=False)  # no interactions -- blow!
+def test_authfail404_noninteractive():
+    # we do not get to the 2nd attempt so just get 'failed'
+    # and exception thrown inside is not emerging all the way here but
+    # caught in the check_
+    check_httpretty_authfail404(['failed'])
+
+
 @skip_if(not httpretty, "no httpretty")
 @without_http_proxy
 @httpretty.activate
-@with_tempfile(mkdir=True)
 @with_fake_cookies_db
-@with_testsui(responses=['yes'])  # will request to reentry it
-def test_HTMLFormAuthenticator_httpretty_authfail404(d):
+@with_tempfile(mkdir=True)
+def check_httpretty_authfail404(exp_called, d):
     # mimic behavior of nersc which 404s but provides feedback whenever
     # credentials are incorrect.  In our case we should fail properly
     credential = FakeCredential1(name='test', url=None)
@@ -412,7 +441,7 @@ def test_HTMLFormAuthenticator_httpretty_authfail404(d):
     downloader = HTTPDownloader(credential=credential, authenticator=authenticator)
     # first one goes with regular DownloadError -- was 404 with not matching content
     assert_raises(DownloadError, downloader.download, url, path=d)
-    assert_equal(was_called, ['failed', '404'])
+    assert_equal(was_called, exp_called)
 
 
 class FakeCredential2(UserPassword):
@@ -431,7 +460,7 @@ class FakeCredential2(UserPassword):
 @httpretty.activate
 @with_tempfile(mkdir=True)
 @with_fake_cookies_db(cookies={'example.com': dict(some_site_id='idsomething', expires='Tue, 15 Jan 2013 21:47:38 GMT')})
-def test_HTMLFormAuthenticator_httpretty_2(d):
+def test_scenario_2(d):
     fpath = opj(d, 'crap.txt')
 
     credential = FakeCredential2(name='test', url=None)
@@ -487,3 +516,137 @@ def test_HTMLFormAuthenticator_httpretty_2(d):
     with open(fpath) as f:
         content = f.read()
         assert_equal(content, "correct body")
+
+
+class FakeCredential3(Token):
+    """Credential to test scenarios."""
+    _fixed_credentials = {'token' : 'testtoken' }
+    def is_known(self):
+        return True
+    def __call__(self):
+        return self._fixed_credentials
+    def enter_new(self):
+        return self._fixed_credentials
+
+@skip_if(not httpretty, "no httpretty")
+@without_http_proxy
+@httpretty.activate
+@with_tempfile(mkdir=True)
+@with_fake_cookies_db
+def test_HTTPBearerTokenAuthenticator(d):
+    fpath = opj(d, 'crap.txt')
+
+    def request_get_callback(request, uri, headers):
+        # We can't assert inside the callback, or running the
+        # test give "Connection aborted" errors instead of telling
+        # us that the assertion failed. So instead, we make
+        # the request object available outside of the callback
+        # and do the assertions in the main test, not the callback
+        request_get_callback.req = request
+        return (200, headers, "correct body")
+
+    httpretty.register_uri(httpretty.GET, url,
+                           body=request_get_callback)
+
+
+
+    credential = FakeCredential3(name='test', url=None)
+    authenticator = HTTPBearerTokenAuthenticator()
+    downloader = HTTPDownloader(credential=credential, authenticator=authenticator)
+    downloader.download(url, path=d)
+
+    # Perform assertions. See note above.
+    r = request_get_callback.req
+    assert_equal(r.body, '')
+    assert_in('Authorization', r.headers)
+    assert_equal(r.headers['Authorization'], "Bearer testtoken")
+
+    with open(fpath) as f:
+        content = f.read()
+        assert_equal(content, "correct body")
+
+class FakeLorisCredential(Token):
+    """Credential to test scenarios."""
+    _fixed_credentials = {'token' : 'testtoken' }
+    def is_known(self):
+        return False
+@skip_if(not httpretty, "no httpretty")
+@without_http_proxy
+@httpretty.activate
+@with_tempfile(mkdir=True)
+@with_fake_cookies_db
+def test_HTTPLorisTokenAuthenticator(d):
+    fpath = opj(d, 'crap.txt')
+
+    def request_get_callback(request, uri, headers):
+        # We can't assert inside the callback, or running the
+        # test give "Connection aborted" errors instead of telling
+        # us that the assertion failed. So instead, we make
+        # the request object available outside of the callback
+        # and do the assertions in the main test, not the callback
+        request_get_callback.req = request
+        return (200, headers, "correct body")
+
+    httpretty.register_uri(httpretty.GET, url,
+                           body=request_get_callback)
+
+
+
+    credential = FakeCredential3(name='test', url=None)
+    authenticator = HTTPBearerTokenAuthenticator()
+    downloader = HTTPDownloader(credential=credential, authenticator=authenticator)
+    downloader.download(url, path=d)
+
+    # Perform assertions. See note above.
+    r = request_get_callback.req
+    assert_equal(r.body, '')
+    assert_in('Authorization', r.headers)
+    assert_equal(r.headers['Authorization'], "Bearer testtoken")
+
+    with open(fpath) as f:
+        content = f.read()
+        assert_equal(content, "correct body")
+
+@skip_if(not httpretty, "no httpretty")
+@without_http_proxy
+@httpretty.activate
+@with_tempfile(mkdir=True)
+@with_fake_cookies_db
+@with_memory_keyring
+@with_testsui(responses=['yes', 'user'])
+def test_lorisadapter(d, keyring):
+    fpath = opj(d, 'crap.txt')
+    loginurl = "http://www.example.com/api/v0.0.2/login"
+
+    def request_get_callback(request, uri, headers):
+        # We can't assert inside the callback, or running the
+        # test give "Connection aborted" errors instead of telling
+        # us that the assertion failed. So instead, we make
+        # the request object available outside of the callback
+        # and do the assertions in the main test, not the callback
+        request_get_callback.req = request
+        return (200, headers, "correct body")
+    def request_post_callback(request, uri, headers):
+        return (200, headers, '{ "token": "testtoken33" }')
+
+    httpretty.register_uri(httpretty.GET, url,
+                           body=request_get_callback)
+    httpretty.register_uri(httpretty.POST, loginurl,
+                           body=request_post_callback)
+
+
+
+    credential = LORIS_Token(name='test', url=loginurl, keyring=None)
+    authenticator = HTTPBearerTokenAuthenticator()
+    downloader = HTTPDownloader(credential=credential, authenticator=authenticator)
+    downloader.download(url, path=d)
+
+    r = request_get_callback.req
+    assert_equal(r.body, '')
+    assert_in('Authorization', r.headers)
+    assert_equal(r.headers['Authorization'], "Bearer testtoken33")
+    # Verify credentials correctly set to test user:pass
+
+    with open(fpath) as f:
+        content = f.read()
+        assert_equal(content, "correct body")
