@@ -41,11 +41,10 @@ _group_dataset = (
          'create-sibling-github'),
         ('datalad.interface.unlock', 'Unlock', 'unlock'),
         ('datalad.interface.save', 'Save', 'save'),
-        ('datalad.plugin', 'Plugin', 'plugin'),
     ])
 
 _group_metadata = (
-    'Commands for meta data handling',
+    'Commands for metadata handling',
     [
         ('datalad.metadata.search', 'Search',
          'search', 'search'),
@@ -53,14 +52,14 @@ _group_metadata = (
          'metadata'),
         ('datalad.metadata.aggregate', 'AggregateMetaData',
          'aggregate-metadata', 'aggregate_metadata'),
+        ('datalad.metadata.extract_metadata', 'ExtractMetadata',
+         'extract-metadata', 'extract_metadata'),
     ])
 
 _group_misc = (
     'Miscellaneous commands',
     [
         ('datalad.interface.test', 'Test'),
-        ('datalad.interface.crawl', 'Crawl'),
-        ('datalad.interface.crawl_init', 'CrawlInit', 'crawl-init'),
         ('datalad.interface.ls', 'Ls'),
         ('datalad.interface.clean', 'Clean'),
         ('datalad.interface.add_archive_content', 'AddArchiveContent',
@@ -68,6 +67,7 @@ _group_misc = (
         ('datalad.interface.download_url', 'DownloadURL', 'download-url'),
         ('datalad.interface.run', 'Run', 'run'),
         ('datalad.interface.rerun', 'Rerun', 'rerun'),
+        ('datalad.interface.run_procedure', 'RunProcedure', 'run-procedure'),
     ])
 
 _group_plumbing = (
@@ -82,3 +82,9 @@ _group_plumbing = (
         ('datalad.support.sshrun', 'SSHRun', 'sshrun'),
         ('datalad.distribution.subdatasets', 'Subdatasets', 'subdatasets'),
     ])
+
+# Some known extensions and their commands to suggest whenever lookup fails
+_known_extension_commands = {
+    'datalad-crawler': ('crawl', 'crawl-init'),
+    'datalad-neuroimaging': ('bids2scidata',)
+}
