@@ -28,7 +28,7 @@ from datalad.distribution.dataset import Dataset
 
 
 lgr = logging.getLogger('datalad.interface.results')
-
+lgr.log(5, "Importing datalad.interface.results")
 
 # which status is a success , which is failure
 success_status_map = {
@@ -178,6 +178,7 @@ known_result_xfms = {
     'successdatasets-or-none': YieldDatasets(success_only=True),
     'paths': YieldField('path'),
     'relpaths': YieldRelativePaths(),
+    'metadata': YieldField('metadata'),
 }
 
 translate_annex_notes = {
@@ -217,11 +218,18 @@ def annexjson2result(d, ds, **kwargs):
         res['action'] = d['command']
     if 'key' in d:
         res['annexkey'] = d['key']
+    if 'fields' in d:
+        # this is annex metadata, filter out timestamps
+        res['metadata'] = {k: v[0] if isinstance(v, list) and len(v) == 1 else v
+                           for k, v in d['fields'].items()
+                           if not k.endswith('lastchanged')}
     # avoid meaningless standard messages
-    if 'note' in d and (
-            d['note'] != 'checksum...' and
-            not d['note'].startswith('checking file')):
-        res['message'] = translate_annex_notes.get(d['note'], d['note'])
+    if 'note' in d:
+        note = "; ".join(ln for ln in d['note'].splitlines()
+                         if ln != 'checksum...'
+                         and not ln.startswith('checking file'))
+        if note:
+            res['message'] = translate_annex_notes.get(note, note)
     return res
 
 
@@ -280,7 +288,8 @@ def is_result_matching_pathsource_argument(res, **kwargs):
 
 
 def results_from_annex_noinfo(ds, requested_paths, respath_by_status, dir_fail_msg,
-                              noinfo_dir_msg, noinfo_file_msg, **kwargs):
+                              noinfo_dir_msg, noinfo_file_msg, noinfo_status='notneeded',
+                              **kwargs):
     """Helper to yield results based on what information git annex did no give us.
 
     The helper assumes that the annex command returned without an error code,
@@ -310,6 +319,8 @@ def results_from_annex_noinfo(ds, requested_paths, respath_by_status, dir_fail_m
     noinfo_file_msg : str
       Message to inject into the result for a requested file that `git
       annex` was silent about.
+    noinfo_status : str
+      Status to report when annex provides no information
     **kwargs
       Any further kwargs are included in the yielded result dictionary.
     """
@@ -346,7 +357,7 @@ def results_from_annex_noinfo(ds, requested_paths, respath_by_status, dir_fail_m
                     fp for fp in respath_by_status.get('success', [])
                     if path_is_subpath(fp, p)]
                 yield get_status_dict(
-                    status='ok' if success_results else 'notneeded',
+                    status='ok' if success_results else noinfo_status,
                     message=None if success_results else (noinfo_dir_msg, p),
                     type='directory', **common_report)
             continue
@@ -355,6 +366,9 @@ def results_from_annex_noinfo(ds, requested_paths, respath_by_status, dir_fail_m
             # yet no exception, hence the file was most probably
             # already in the desired state
             yield get_status_dict(
-                status='notneeded', type='file',
+                status=noinfo_status, type='file',
                 message=noinfo_file_msg,
                 **common_report)
+
+
+lgr.log(5, "Done importing datalad.interface.results")
\ No newline at end of file
