@@ -15,7 +15,9 @@ import logging.handlers
 
 from os.path import basename, dirname
 
-from .utils import is_interactive
+from collections import defaultdict
+
+from .utils import is_interactive, optional_args
 from .support import ansi_colors as colors
 
 __all__ = ['ColorFormatter']
@@ -65,7 +67,7 @@ class TraceBack(object):
         self._extract_stack = traceback.extract_stack
 
     def __call__(self):
-        ftb = self._extract_stack(limit=200)[:-2]
+        ftb = self._extract_stack(limit=self.limit+10)[:-2]
         entries = [[mbasename(x[0]), str(x[1])]
                    for x in ftb if mbasename(x[0]) != 'logging.__init__']
         entries = [e for e in entries if e[0] != 'unittest']
@@ -104,6 +106,37 @@ class TraceBack(object):
         return sftb
 
 
+class MemoryInfo(object):
+    def __init__(self):
+        try:
+            from psutil import Process
+            process = Process(os.getpid())
+            self.memory_info = process.memory_info \
+                if hasattr(process, 'memory_info') \
+                else process.get_memory_info
+        except:
+            self.memory_info = None
+
+
+    def __call__(self):
+        """Return utilization of virtual memory
+
+        Generic implementation using psutil
+        """
+        if not self.memory_info:
+            return "RSS/VMS: N/A"
+        mi = self.memory_info()
+        # in later versions of psutil mi is a named tuple.
+        # but that is not the case on Debian squeeze with psutil 0.1.3
+        rss = mi[0] / 1024
+        vms = mi[1] / 1024
+        vmem = (rss, vms)
+
+        try:
+            return "RSS/VMS: %d/%d kB" % vmem
+        except:
+            return "RSS/VMS: %s" % str(vmem)
+
 # Recipe from http://stackoverflow.com/questions/384076/how-can-i-color-python-logging-output
 # by Brandon Thomson
 # Adjusted for automagic determination either coloring is needed and
@@ -121,6 +154,8 @@ class ColorFormatter(logging.Formatter):
         collide = log_env == 'collide'
         limit = 100 if collide else int(log_env) if log_env.isdigit() else 100
         self._tb = TraceBack(collide=collide, limit=limit) if log_env else None
+
+        self._mem = MemoryInfo() if os.environ.get('DATALAD_LOG_VMEM', '') else None
         logging.Formatter.__init__(self, msg)
 
     def _get_format(self, log_name=False, log_pid=False):
@@ -148,11 +183,158 @@ class ColorFormatter(logging.Formatter):
             record.levelname = levelname_color
         record.msg = record.msg.replace("\n", "\n| ")
         if self._tb:
-            record.msg = self._tb() + "  " + record.msg
+            if not getattr(record, 'notraceback', False):
+                record.msg = self._tb() + "  " + record.msg
+        if self._mem:
+            record.msg = "%s %s" % (self._mem(), record.msg)
 
         return logging.Formatter.format(self, record)
 
 
+class ProgressHandler(logging.Handler):
+    from datalad.ui import ui
+
+    def __init__(self):
+        super(self.__class__, self).__init__()
+        self.pbars = {}
+
+    def emit(self, record):
+        from datalad.ui import ui
+        pid = getattr(record, 'dlm_progress')
+        update = getattr(record, 'dlm_progress_update', None)
+        # would be an actual message, not used ATM here,
+        # and the record not passed to generic handler ATM
+        # (filtered away by NoProgressLog)
+        # so no final message is printed
+        # msg = record.getMessage()
+        if pid not in self.pbars:
+            # this is new
+            pbar = ui.get_progressbar(
+                label=getattr(record, 'dlm_progress_label', ''),
+                unit=getattr(record, 'dlm_progress_unit', ''),
+                total=getattr(record, 'dlm_progress_total', None))
+            self.pbars[pid] = pbar
+        elif update is None:
+            # not an update -> done
+            # TODO if the other logging that is happening is less frontpage
+            # we may want to actually "print" the completion message
+            self.pbars.pop(pid).finish()
+        else:
+            # an update
+            self.pbars[pid].update(
+                update,
+                increment=getattr(record, 'dlm_progress_increment', False))
+            # Check for an updated label.
+            label = getattr(record, 'dlm_progress_label', None)
+            if label is not None:
+                self.pbars[pid].set_desc(label)
+
+
+class NoProgressLog(logging.Filter):
+    def filter(self, record):
+        return not hasattr(record, 'dlm_progress')
+
+
+class OnlyProgressLog(logging.Filter):
+    def filter(self, record):
+        return hasattr(record, 'dlm_progress')
+
+
+def log_progress(lgrcall, pid, *args, **kwargs):
+    """Helper to emit a log message on the progress of some process
+
+    Parameters
+    ----------
+    lgrcall : callable
+      Something like lgr.debug or lgr.info
+    pid : str
+      Some kind of ID for the process the progress is reported on.
+    *args : str
+      Log message, and potential arguments
+    total : int
+      Max progress quantity of the process.
+    label : str
+      Process description. Should be very brief, goes in front of progress bar
+      on the same line.
+    unit : str
+      Progress report unit. Should be very brief, goes after the progress bar
+      on the same line.
+    update : int
+      To which quantity to advance the progress.
+    incremental : bool
+      If set, `update` is interpreted as an incremental value, not absolute.
+    """
+    d = dict(
+        {'dlm_progress_{}'.format(n): v for n, v in kwargs.items()
+         if v},
+        dlm_progress=pid)
+    lgrcall(*args, extra=d)
+
+
+@optional_args
+def with_result_progress(fn, label="Total", unit=" Files"):
+    """Wrap a progress bar, with status counts, around a function.
+
+    Parameters
+    ----------
+    fn : generator function
+        This function should accept a collection of items as a
+        positional argument and any number of keyword arguments.  After
+        processing each item in the collection, it should yield a status
+        dict.
+    label, unit : str
+        Passed to log.log_progress.
+
+    Returns
+    -------
+    A variant of `fn` that shows a progress bar.  Note that the wrapped
+    function is not a generator function; the status dicts will be
+    returned as a list.
+    """
+    # FIXME: This emulates annexrepo.ProcessAnnexProgressIndicators.  It'd be
+    # nice to rewire things so that it could be used directly.
+
+    def count_str(count, verb, omg=False):
+        if count:
+            msg = "{:d} {}".format(count, verb)
+            if omg:
+                msg = colors.color_word(msg, colors.RED)
+            return msg
+
+    pid = str(fn)
+    base_label = label
+
+    def wrapped(items, **kwargs):
+        counts = defaultdict(int)
+
+        label = base_label
+        log_progress(lgr.info, pid,
+                     "%s: starting", label,
+                     total=len(items), label=label, unit=unit)
+
+        results = []
+        for res in fn(items, **kwargs):
+            counts[res["status"]] += 1
+            count_strs = (count_str(*args)
+                          for args in [(counts["notneeded"], "skipped", False),
+                                       (counts["error"], "failed", True)])
+            if counts["notneeded"] or counts["error"]:
+                label = "{} ({})".format(
+                    base_label,
+                    ", ".join(filter(None, count_strs)))
+
+            log_progress(
+                lgr.error if res["status"] == "error" else lgr.info,
+                pid,
+                "%s: processed result%s", base_label,
+                " for " + res["path"] if "path" in res else "",
+                label=label, update=1, increment=True)
+            results.append(res)
+        log_progress(lgr.info, pid, "%s: done", base_label)
+        return results
+    return wrapped
+
+
 class LoggerHelper(object):
     """Helper to establish and control a Logger"""
 
@@ -268,7 +450,16 @@ class LoggerHelper(object):
         #  logging.Formatter('%(asctime)-15s %(levelname)-6s %(message)s'))
         self.lgr.addHandler(loghandler)
 
+        if is_interactive():
+            phandler = ProgressHandler()
+            # progress only when interactive
+            phandler.addFilter(OnlyProgressLog())
+            # no stream logs of progress messages when interactive
+            loghandler.addFilter(NoProgressLog())
+            self.lgr.addHandler(phandler)
+
         self.set_level()  # set default logging level
         return self.lgr
 
+
 lgr = LoggerHelper().get_initialized_logger()
