@@ -10,8 +10,14 @@
 """
 
 
-from io import open
+import io
 import codecs
+from six import PY2
+from os.path import dirname
+from os.path import exists
+from os import makedirs
+import os
+import os.path as op
 
 # wrapped below
 from simplejson import load as jsonload
@@ -21,8 +27,20 @@ from simplejson import loads as json_loads
 from simplejson import JSONDecodeError
 
 
-# TODO think about minimizing the JSON output by default
-json_dump_kwargs = dict(indent=2, sort_keys=True, ensure_ascii=False, encoding='utf-8')
+# produce relatively compact, but also diff-friendly format
+json_dump_kwargs = dict(
+    indent=0,
+    separators=(',', ':\n'),
+    sort_keys=True,
+    ensure_ascii=False,
+    encoding='utf-8', )
+
+# achieve minimal representation, but still deterministic
+compressed_json_dump_kwargs = dict(
+    json_dump_kwargs,
+    indent=None,
+    separators=(',', ':'))
+
 
 # Let's just reuse top level one for now
 from ..log import lgr
@@ -30,11 +48,65 @@ from ..dochelpers import exc_str
 
 
 def dump(obj, fname):
-    with open(fname, 'wb') as f:
-        return jsondump(
-            obj,
-            codecs.getwriter('utf-8')(f),
-            **json_dump_kwargs)
+    indir = dirname(fname)
+    if not exists(indir):
+        makedirs(indir)
+    with io.open(fname, 'wb') as f:
+        return dump2fileobj(obj, f)
+
+
+def dump2fileobj(obj, fileobj):
+    return jsondump(
+        obj,
+        codecs.getwriter('utf-8')(fileobj),
+        **json_dump_kwargs)
+
+
+def LZMAFile(*args, **kwargs):
+    """A little decorator to overcome a bug in lzma
+
+    A unique to yoh and some others bug with pyliblzma
+    calling dir() helps to avoid AttributeError __exit__
+    see https://bugs.launchpad.net/pyliblzma/+bug/1219296
+    """
+    from .lzma import lzma
+    lzmafile = lzma.LZMAFile(*args, **kwargs)
+    dir(lzmafile)
+    return lzmafile
+
+
+def dump2stream(obj, fname, compressed=False):
+
+    _open = LZMAFile if compressed else open
+
+    indir = dirname(fname)
+
+    if op.lexists(fname):
+        os.remove(fname)
+    elif indir and not exists(indir):
+        makedirs(indir)
+    with _open(fname, mode='wb') as f:
+        jwriter = codecs.getwriter('utf-8')(f)
+        for o in obj:
+            jsondump(o, jwriter, **compressed_json_dump_kwargs)
+            f.write(b'\n')
+
+
+def dump2xzstream(obj, fname):
+    dump2stream(obj, fname, compressed=True)
+
+
+def load_stream(fname, compressed=False):
+
+    _open = LZMAFile if compressed else open
+    with _open(fname, mode='r') as f:
+        for line in f:
+            yield loads(line)
+
+
+def load_xzstream(fname):
+    for o in load_stream(fname, compressed=True):
+        yield o
 
 
 def loads(s, *args, **kwargs):
@@ -60,7 +132,7 @@ def load(fname, fixup=True, **kw):
     **kw
       Passed into the load (and loads after fixups) function
     """
-    with open(fname, 'r', encoding='utf-8') as f:
+    with io.open(fname, 'r', encoding='utf-8') as f:
         try:
             return jsonload(f, **kw)
         except JSONDecodeError as exc:
@@ -70,7 +142,7 @@ def load(fname, fixup=True, **kw):
 
     # Load entire content and replace common "abusers" which break JSON comprehension but in general
     # are Ok
-    with open(fname, 'r', encoding='utf-8') as f:
+    with io.open(fname, 'r', encoding='utf-8') as f:
         s_orig = s = f.read()
 
     for o, r in {
