@@ -546,7 +546,8 @@ def __test_get_md5s(path):
     # was used just to generate above dict
     annex = AnnexRepo(path, init=True, backend='MD5E')
     files = [basename(f) for f in find_files('.*', path)]
-    annex.add(files, commit=True)
+    annex.add(files)
+    annex.commit()
     print({f: annex.get_file_key(f) for f in files})
 
 
@@ -556,7 +557,8 @@ def test_dropkey(batch, direct, path):
     kw = {'batch': batch}
     annex = AnnexRepo(path, init=True, backend='MD5E', direct=direct)
     files = list(tree1_md5e_keys)
-    annex.add(files, commit=True)
+    annex.add(files)
+    annex.commit()
     # drop one key
     annex.drop_key(tree1_md5e_keys[files[0]], **kw)
     # drop multiple
@@ -768,7 +770,8 @@ def test_AnnexRepo_add_to_annex(path):
     with open(opj(repo.path, filename), "w") as f:
         f.write("something else")
 
-    repo.add(filename, commit=True, msg="Added another file to annex.")
+    repo.add(filename)
+    repo.commit(msg="Added another file to annex.")
     # known to annex:
     ok_(repo.get_file_key(filename))
     ok_(repo.file_has_content(filename))
@@ -806,8 +809,8 @@ def test_AnnexRepo_add_to_git(path):
     with open(opj(repo.path, filename), "w") as f:
         f.write("something else")
 
-    repo.add(filename, git=True, commit=True,
-             msg="Added another file to annex.")
+    repo.add(filename, git=True)
+    repo.commit(msg="Added another file to annex.")
     # not in annex, but in git:
     assert_raises(FileInGitError, repo.get_file_key, filename)
 
@@ -930,6 +933,10 @@ def test_AnnexRepo_get_contentlocation():
 @with_tempfile
 def test_AnnexRepo_addurl_to_file_batched(sitepath, siteurl, dst):
 
+    if os.environ.get('DATALAD_FAKE__DATES'):
+        raise SkipTest(
+            "Faked dates are enabled; skipping batched addurl tests")
+
     ar = AnnexRepo(dst, create=True)
     testurl = urljoin(siteurl, 'about.txt')
     testurl2 = urljoin(siteurl, 'about2.txt')
@@ -1026,6 +1033,30 @@ def test_AnnexRepo_addurl_to_file_batched(sitepath, siteurl, dst):
     # TODO: verify that file is added with that backend and that we got a new batched process
 
 
+@with_tree(tree={"foo": "foo content"})
+@serve_path_via_http()
+@with_tree(tree={"bar": "bar content"})
+def test_annexrepo_fake_dates_disables_batched(sitepath, siteurl, dst):
+    ar = AnnexRepo(dst, create=True, fake_dates=True)
+
+    with swallow_logs(new_level=logging.DEBUG) as cml:
+        ar.add_url_to_file("foo-dst", urljoin(siteurl, "foo"), batch=True)
+        cml.assert_logged(
+            msg="Not batching addurl call because fake dates are enabled",
+            level="DEBUG",
+            regex=False)
+
+    ar.add("bar")
+    ar.commit("add bar")
+
+    with swallow_logs(new_level=logging.DEBUG) as cml:
+        ar.drop_key(ar.get_file_key(["bar"]), batch=True)
+        cml.assert_logged(
+            msg="Not batching drop_key call because fake dates are enabled",
+            level="DEBUG",
+            regex=False)
+
+
 @with_tempfile(mkdir=True)
 def test_annex_backends(path):
     repo = AnnexRepo(path)
@@ -1299,6 +1330,34 @@ def test_annex_drop(src, dst):
     assert_raises(CommandError, ar.drop, ['.'], options=['--all'])
 
 
+@with_tree({"a.txt": "a", "b.txt": "b", "c.py": "c", "d": "d"})
+def test_annex_get_annexed_files(path):
+    repo = AnnexRepo(path)
+    repo.add(".")
+    repo.commit()
+    eq_(set(repo.get_annexed_files()), {"a.txt", "b.txt", "c.py", "d"})
+
+    repo.drop("a.txt", options=["--force"])
+    eq_(set(repo.get_annexed_files()), {"a.txt", "b.txt", "c.py", "d"})
+    eq_(set(repo.get_annexed_files(with_content_only=True)),
+        {"b.txt", "c.py", "d"})
+
+    eq_(set(repo.get_annexed_files(patterns=["*.txt"])),
+        {"a.txt", "b.txt"})
+    eq_(set(repo.get_annexed_files(with_content_only=True,
+                                   patterns=["*.txt"])),
+        {"b.txt"})
+
+    eq_(set(repo.get_annexed_files(patterns=["*.txt", "*.py"])),
+        {"a.txt", "b.txt", "c.py"})
+
+    eq_(set(repo.get_annexed_files()),
+        set(repo.get_annexed_files(patterns=["*"])))
+
+    eq_(set(repo.get_annexed_files(with_content_only=True)),
+        set(repo.get_annexed_files(with_content_only=True, patterns=["*"])))
+
+
 @with_testrepos('basic_annex', flavors=['clone'])
 def test_annex_remove(path):
     repo = AnnexRepo(path, create=False)
@@ -1842,7 +1901,8 @@ def _test_status(ar):
     eq_(stat, ar.get_status())
 
     # add to subrepo
-    sub.add('fourth', commit=True, msg="birther mod init'ed")
+    sub.add('fourth')
+    sub.commit(msg="birther mod init'ed")
     stat['untracked'].remove(opj('submod', 'fourth'))
 
     if ar.get_active_branch().endswith('(unlocked)') and \
@@ -2118,6 +2178,17 @@ def test_AnnexRepo_metadata(path):
     eq_(['best'], dict(ar.get_metadata(playfile))[playfile]['novel'])
 
 
+@with_tree(tree={'file.txt': 'content'})
+@serve_path_via_http()
+@with_tempfile
+def test_AnnexRepo_addurl_batched_and_set_metadata(path, url, dest):
+    ar = AnnexRepo(dest, create=True)
+    fname = "file.txt"
+    ar.add_url_to_file(fname, urljoin(url, fname), batch=True)
+    list(ar.set_metadata(fname, init={"number": "one"}))
+    eq_(["one"], dict(ar.get_metadata(fname))[fname]["number"])
+
+
 @with_tempfile(mkdir=True)
 def test_change_description(path):
     # prelude
@@ -2191,3 +2262,14 @@ def test_fake_is_not_special(path):
     # doesn't exist -- we fail by default
     assert_raises(RemoteNotAvailableError, ar.is_special_annex_remote, "fake")
     assert_false(ar.is_special_annex_remote("fake", check_if_known=False))
+
+
+@with_tempfile(mkdir=True)
+def test_fake_dates(path):
+    ar = AnnexRepo(path, create=True, fake_dates=True)
+    timestamp = ar.config.obtain("datalad.fake-dates-start") + 1
+    # Commits from the "git annex init" call are one second ahead.
+    for commit in ar.get_branch_commits("git-annex"):
+        eq_(timestamp, commit.committed_date)
+    assert_in("timestamp={}s".format(timestamp),
+              ar.repo.git.cat_file("blob", "git-annex:uuid.log"))
