@@ -33,6 +33,7 @@ from nose import SkipTest
 
 from ..utils import getpwd, chpwd
 
+from . import utils
 from .utils import eq_, ok_, assert_false, ok_startswith, nok_startswith, \
     with_tempfile, with_testrepos, with_tree, \
     rmtemp, \
@@ -44,6 +45,7 @@ from .utils import eq_, ok_, assert_false, ok_startswith, nok_startswith, \
 
 from .utils import ok_generator
 from .utils import assert_dict_equal
+from .utils import assert_str_equal
 from .utils import assert_re_in
 from .utils import local_testrepo_flavors
 from .utils import skip_if_no_network
@@ -53,6 +55,10 @@ from .utils import skip_if
 from .utils import ok_file_has_content
 from .utils import without_http_proxy
 from .utils import with_testsui
+from .utils import skip_ssh
+from .utils import probe_known_failure
+from .utils import patch_config
+from .utils import ignore_nose_capturing_stdout
 
 #
 # Test with_tempfile, especially nested invocations
@@ -563,6 +569,15 @@ def test_assert_dict_equal():
     assert_raises(AssertionError, assert_dict_equal, {1: 0}, {1: np.arange(3)})
 
 
+def test_assert_str_equal():
+    assert_str_equal("a", "a")
+    assert_str_equal("a\n", "a\n")
+    assert_str_equal("a\nb", "a\nb")
+    assert_raises(AssertionError, assert_str_equal, "a", "a\n")
+    assert_raises(AssertionError, assert_str_equal, "a", "b")
+    assert_raises(AssertionError, assert_str_equal, "ab", "b")
+
+
 def test_testsui():
     # just one for now to test conflicting arguments
     with assert_raises(ValueError):
@@ -593,4 +608,29 @@ def test_setup():
     from datalad.consts import DATASETS_TOPURL
     eq_(DATASETS_TOPURL, 'http://datasets-tests.datalad.org/')
     from datalad.tests.utils import get_datasets_topdir
-    eq_(get_datasets_topdir(), 'datasets-tests.datalad.org')
\ No newline at end of file
+    eq_(get_datasets_topdir(), 'datasets-tests.datalad.org')
+
+
+def test_skip_ssh():
+    with patch_config({'datalad.tests.ssh': False}):
+        with assert_raises(SkipTest):
+            skip_ssh(lambda: False)()
+
+
+def test_probe_known_failure():
+    # should raise assert error if function no longer fails
+    with patch_config({'datalad.tests.knownfailures.probe': True}):
+        with assert_raises(AssertionError):
+            probe_known_failure(lambda: True)()
+
+    with patch_config({'datalad.tests.knownfailures.probe': False}):
+        ok_(probe_known_failure(lambda: True))
+
+
+def test_ignore_nose_capturing_stdout():
+    # Just test the logic, not really a situation under overwritten stdout
+    def raise_exc():
+        raise AttributeError('nose causes a message which includes words '
+                             'StringIO and fileno')
+    with assert_raises(SkipTest):
+        ignore_nose_capturing_stdout(raise_exc)()
